//-*-c++-*-

#ifndef _file_h
#define _file_h

#if defined(__BORLANDC__) || defined(__CYGWIN__) || defined(WIN32)
#define FOPEN_READ_MODE "rb"
#define FOPEN_WRITE_MODE "wb"
#else
#define FOPEN_READ_MODE "r"
#define FOPEN_WRITE_MODE "w"
#endif

#if defined( __cplusplus ) || defined( c_plusplus )

#include <errno.h>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include "utils.h"
// Undo the damage done by Lynxos to errno
//#undef errno
//extern int errno;

#if defined(__BORLANDC__) || defined(WIN32)

#include <wtypes.h>
#include <winbase.h>
#include <winsock.h>

#else

typedef int SOCKET;
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

#endif


#if defined(__BORLANDC__ ) || defined(__CYGWIN__) || defined(WIN32)
#define IOS_READ_MODE (std::ios::in | std::ios::binary)
#define IOS_WRITE_MODE (std::ios::out | std::ios::binary)
#else
#define IOS_READ_MODE (std::ios::in)
#define IOS_WRITE_MODE (std::ios::out)
#endif


#if defined(VXINT)

class fifo
{
protected:

   int fd;

public:

   fifo( const char *path, unsigned mode );
   inline int getfd(void){ return fd; }
};

#endif

// Attempt to make a common exchange IO interface for different stream types
// (file, socket...)

class xiostream
{
protected:

  char *name;
  bool mustSwap4;
  bool mustSwap2;

public:

  tracestream errors;

  xiostream()
  {
    mustSwap4 = ( 0x1234abcd != swap( (unsigned)0x1234abcd ) );
    mustSwap2 = ( 0x1234 != swap( (unsigned short)0x1234 ) );
  }
  virtual ~xiostream(){;}
  operator void*(){ return errors.tellp() ? 0 : this; }
  bool operator !(){ return errors.tellp() <= 0 ? false : true; }

  inline unsigned swap( unsigned );
  inline unsigned short swap( unsigned short );
  virtual void open( const char *nm ) = 0;
  virtual void close(void) = 0;
  virtual int read(void *buf, unsigned len) = 0;
  virtual int write(const void *buf, unsigned len) = 0;
  virtual void flush(void) = 0;
  inline void unlink(void)
  {
#if defined(__CYGWIN__)
    ::unlink( name );
#endif
    return;
  }
  inline xiostream &put(unsigned short);
  inline xiostream &put(unsigned int);
  inline xiostream &get(unsigned short &);
  inline xiostream &get(unsigned int &);
  inline xiostream &put(short d){ put((unsigned short &)d); return *this; }
  inline xiostream &put(int d){ put((unsigned &)d); return *this; }
  inline xiostream &get(short &d){ get((unsigned short &)d); return *this; }
  inline xiostream &get(int &d){ get((unsigned &)d); return *this; }

};

inline unsigned xiostream::swap( unsigned i )
{
union { unsigned char c[4]; unsigned u; } d;

   d.u = i;

   i = d.c[0];
   i |= ( d.c[1] << 8 );
   i |= ( d.c[2] << 16 );
   i |= ( d.c[3] << 24 );

   return i;

}

inline unsigned short xiostream::swap( unsigned short i )
{
union { unsigned char c[2]; unsigned short u; } d;

   d.u = i;

   i = d.c[0];
   i |= ( d.c[1] << 8 );

   return i;

}

inline xiostream &xiostream::get( unsigned short &s )
{
unsigned char c[2];
 
   read( (char *)&c, 2 );

   if ( mustSwap2 )
   {
     s = c[0];
     s |= ( c[1] << 8 );
   }
   else
   {
     s = *(unsigned short *)&c;
   }

   return *this;

}

inline xiostream &xiostream::put( unsigned short s )
{
unsigned char c[2];

   if ( mustSwap2 )
   {
     c[0] = s & 0xff;
     c[1] = ( s >> 8 ) & 0xff;
   }
   else
   {
     *(unsigned short *)&c = s;
   }

   write( (char *)&c, 2 );

   return *this;

}

inline xiostream &xiostream::get( unsigned int &s )
{
unsigned char c[4];
 
   read( (char *)&c, 4 );

   if ( mustSwap4 )
   {
     s = c[0];
     s |= ( c[1] << 8 );
     s |= ( c[2] << 16 );
     s |= ( c[3] << 24 );
   }
   else
   {
     s = *(unsigned int *)&c;
   }

   return *this;

}

inline xiostream &xiostream::put( unsigned int s )
{
unsigned char c[4];

   if ( mustSwap4 )
   {
     c[0] = s & 0xff;
     c[1] = ( s >> 8 ) & 0xff;
     c[2] = ( s >> 16 ) & 0xff;
     c[3] = ( s >> 24 ) & 0xff;
   }
   else
   {
     *(unsigned int *)&c = s;
   }

   write( (char *)&c, 4 );

   return *this;

}

class xfstream: public xiostream
{
protected:

  std::ios_base::openmode iosmode;
  std::fstream f;

public:

  xfstream( const char *nm=0, std::ios_base::openmode md=std::ios::in );
  ~xfstream(){;}
  
  inline void open(const char *nm=0);
  void close(void){ f.close(); return; }
  int read(void *buf, unsigned len){ f.read((char *)buf,len); return 0; }
  int write(const void *buf, unsigned len){ f.write((char *)buf,len); return 0; }
  inline void flush(void){ f.flush(); return; }
};

inline xfstream::xfstream( const char *nm, std::ios_base::openmode md ):xiostream(),f()
{
   iosmode = md;

   if ( !nm ) return;

   char *fn = getenv(nm);
   if ( fn )
   {
      name = new char[strlen(fn)+1];
      strcpy( name,fn );
   }
   else
   {
      name = new char[strlen(nm)+1];
      strcpy( name,nm );
   }

   f.open( name, iosmode );

   return;
}

inline void xfstream::open(const char *nm)
{
  if ( nm )
  {
    delete [] name;
    char *fn = getenv(nm);
    if ( fn )
    {
      name = new char[strlen(fn)+1];
      strcpy( name,fn );
    }
    else
    {
      name = new char[strlen(nm)+1];
      strcpy( name,nm );
    }
  }

  f.open(name, iosmode );

  return;
}

class xsrvsock
{
friend class xsockstream;

protected:

   char *host;
   unsigned short port;
   SOCKET ls;

public:

   tracestream errors;

   xsrvsock( unsigned short pt );
   operator void*(){ return errors.tellp() ? 0 : this; }

};


class xsockstream: public xiostream
{
protected:

  SOCKET s;

public:

  inline xsockstream():xiostream(){ return; }
  inline xsockstream(SOCKET sk){ s = sk; return; }
  inline xsockstream(const char *host, unsigned port);
  xsockstream(xsrvsock *);
  inline ~xsockstream(){ return; }
  
  inline void open(const char *nm){ return; }
  inline void close(void){ ::close(s); return; }
  inline int read(void *buf, unsigned len){ return recv(s,buf,len,0); }
  inline int write(const void *buf, unsigned len){ return send(s,buf,len,0); }
  inline void flush(void){ return; }
  inline void setHandle(int sk){ s=(SOCKET)sk; return; }
};

inline xsockstream::xsockstream(const char *host, unsigned port):xiostream()
{
 return;
}


class envfstream: public std::fstream
{

public:

   enum
   {
      maxBufferLength = 512
   };

   char *name;
   char mustSwap4;
   char mustSwap2;

   inline envfstream( const char *, std::ios_base::openmode md = std::ios::in );
  //   inline envfstream( int fd, int md = std::ios::in, int prot=0664 );
   ~envfstream(){ delete name; }
   inline void init( std::ios_base::openmode );
   inline void open( const char *, std::ios_base::openmode );
   inline envfstream &operator<<(unsigned short);
   inline envfstream &operator<<(unsigned int);
   inline envfstream &operator>>(unsigned short &);
   inline envfstream &operator>>(unsigned int &);
   inline envfstream &operator<<(short d);
   inline envfstream &operator<<(int d);
   inline envfstream &operator>>(short &d);
   inline envfstream &operator>>(int &d);
   inline unsigned iin( unsigned );
   inline unsigned short iin( unsigned short );
   inline unsigned iout( unsigned );
   inline unsigned short iout( unsigned short );
#if defined( VXINT )
   inline envfstream &scan( const char *, ... );
   inline envfstream &form( const char *, ... );
#endif

protected:

   std::ios_base::openmode iosmode;

};

inline unsigned envfstream::iin( unsigned i )
{
union { unsigned char c[4]; unsigned u; } d;

   d.u = i;

   i = d.c[0];
   i |= ( d.c[1] << 8 );
   i |= ( d.c[2] << 16 );
   i |= ( d.c[3] << 24 );

   return i;

}

inline unsigned short envfstream::iin( unsigned short i )
{
union { unsigned char c[2]; unsigned short u; } d;

   d.u = i;

   i = d.c[0];
   i |= ( d.c[1] << 8 );

   return i;

}

inline unsigned envfstream::iout( unsigned i )
{
unsigned char c[4];

   c[0] = i & 0xff;
   c[1] = ( i >> 8 ) & 0xff;
   c[2] = ( i >> 16 ) & 0xff;
   c[3] = ( i >> 24 ) & 0xff;

   return *(unsigned *)c;

}

inline unsigned short envfstream::iout( unsigned short s )
{
unsigned char c[2];

   c[0] = s & 0xff;
   c[1] = ( s >> 8 ) & 0xff;

   return *(unsigned short *)c;

}

inline envfstream::envfstream( const char *nm, std::ios_base::openmode md ):std::fstream()
{
   init(md);
   if ( !nm ) return;

   open( nm, iosmode );

   return;
}

//inline envfstream::envfstream( int fd, int md, int prot ):std::fstream(fd)
//{
//   init(md,prot);
//}

void envfstream::init( std::ios_base::openmode md )
{
   mustSwap4 = ( 0x1234abcd != iin( (unsigned)0x1234abcd ) );
   mustSwap2 = ( 0x1234 != iin( (unsigned short)0x1234 ) );

   iosmode = md;

   return;
}

inline void envfstream::open( const char *nm, std::ios_base::openmode md )
{
   char *fn = getenv(nm);
   if ( fn )
   {
      name = new char[strlen(fn)+1];
      strcpy( name,fn );
   }
   else
   {
      name = new char[strlen(nm)+1];
      strcpy( name,nm );
   }

   std::fstream::open( name, md );

   return;
}

inline envfstream &envfstream::operator>>( unsigned short &s )
{
unsigned char c[2];
 
   read( (char *)&c, 2 );
 
   s = c[0];
   s |= ( c[1] << 8 );

   return *this;

}

inline envfstream &envfstream::operator<<( unsigned short s )
{
unsigned char c[2];

   c[0] = s & 0xff;
   c[1] = ( s >> 8 ) & 0xff;

   write( (char *)&c, 2 );

   return *this;

}

inline envfstream &envfstream::operator>>( unsigned int &s )
{
unsigned char c[4];
 
   read( (char *)&c, 4 );

   s = c[0];
   s |= ( c[1] << 8 );
   s |= ( c[2] << 16 );
   s |= ( c[3] << 24 );

   return *this;

}

inline envfstream &envfstream::operator<<( unsigned int s )
{
unsigned char c[4];

   c[0] = s & 0xff;
   c[1] = ( s >> 8 ) & 0xff;
   c[2] = ( s >> 16 ) & 0xff;
   c[3] = ( s >> 24 ) & 0xff;

   write( (char *)&c, 4 );

   return *this;

}

inline envfstream &envfstream::operator<<(short d){ *this << (unsigned short &)d; return *this; }
inline envfstream &envfstream::operator<<(int d){ *this << (unsigned &)d; return *this; }
inline envfstream &envfstream::operator>>(short &d){ *this >> (unsigned short &)d; return *this; }
inline envfstream &envfstream::operator>>(int &d){ *this >> (unsigned &)d; return *this; }

#if defined( VXINT )

inline envfstream &envfstream::scan( const char *fmt, ... )
{
va_list ap;
char buf[maxBufferLength+1];

   getline( buf, maxBufferLength );

   va_start( ap,fmt );
   vsscanf( buf, fmt, ap );
   va_end( ap );

   return *this;
}

inline envfstream &envfstream::form( const char *fmt, ... )
{
va_list ap;
char buf[maxBufferLength+1];

   va_start( ap,fmt );
   int rc = vsprintf( buf, fmt, ap );
   va_end( ap );

   if ( rc != EOF ) write( buf, rc );

   return *this;
}
#endif

#endif

#endif
