/*
pbwrelay server process

Steve Wotton
Based on emurelay

This process is forked from the pbwrelay daemon (pbwrelayd) process
and handles the socket connection on behalf of a single remote client.
This process inherits the socket file descriptor as stdout and the
pipe from the pbwrelay daemon as stdin.

This server enters a loop reading messages from the daemon and
forwards the selected ones to the client.

Any errors cause the process to exit which should allow the daemon to
clean up.

*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>
#include <syslog.h>

#include "tcp.h"

#define MAX_MESSAGE_LENGTH 8192

int main( int argc, char **argv )
{
int rc,s,Length;
char Message[MAX_MESSAGE_LENGTH+1];
int readfds;

/* The socket is now stdout */

   s = 1;
   sockstream *ss = new sockstream(s);

   fcntl( 0, F_SETFL, O_NONBLOCK );
   
/* Loop forever */

   for ( ;; )
   {

/* Check for any IO activity */

      do
      {
         errno = 0;
         readfds = 1<<0 | 1<<s;
         select( 32, (fd_set *)&readfds, 0, 0, 0 );
      } while ( errno == EINTR );
      
/* If the socket, s, has a read pending, this must be the result of
the client closing it. Close down if this is the case or for other
unrecoverable errors. */

      if ( errno || (readfds & (1<<s)) )
      {
         syslog( LOG_ERR | LOG_USER, "pbwrelay select(): break\n" );
         if (errno) syslog( LOG_ERR | LOG_USER, "pbwrelay select(): %m\n" );
         break;
      }
      
/* Any other pending read must be from daemon. Messages are null terminated. */

      for(;;)
      {
         Length = msg_rcv( 0, Message, MAX_MESSAGE_LENGTH );
         if ( Length == 0 ) break;
         if ( Length < 0 )
         {
            syslog( LOG_ERR | LOG_USER, "pbwrelay:msg_rcv(): error\n" );
            exit(0);
	 }
      
/* Send the message to the remote client */

         unsigned len = *reinterpret_cast<unsigned *>(Message);
         try { ss->send(Message,len); }
         catch ( ss_exception &sse )
         {
           syslog(LOG_ERR | LOG_USER,"pbwrelay:ss->send() %s errcode=%d\n",sse.what(),sse.errcode());
           break;
         }
      }
   }

   syslog( LOG_ERR | LOG_USER, "pbwrelay:exit():\n" );
   exit(0);

}
