/* %Q% */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libtcp.h"

int msg_rcv( int fd, void **buf, int size )
{
int len, rcvd, cc, packets, remainder;
unsigned char clen[4];
char *msgbuf;

/* Read the length */

   for( rcvd=0; rcvd<4; rcvd+=cc )
   {
      errno = 0;

      cc = read( fd, (char*)clen + rcvd, 4 - rcvd );

      if ( cc == 0 ) return -1; /* EOF (i.e. socket closed) condition */

      if ( cc < 0 )
      {
         if ( errno == EAGAIN && !rcvd ) return 0;
         else if ( errno == EINTR || errno == EAGAIN ) cc = 0;
         else return -1;
      }
   }

/* Decode the big-endian length to machine representation */

   len = clen[3];
   len |= clen[2]<<8;
   len |= clen[1]<<16;
   len |= clen[0]<<24;

/* Check the buffer is large enough or create one if needed */

   if ( size == 0 )
   {
      msgbuf = malloc( len );
      if ( !msgbuf ) return -1;
      *buf = msgbuf;
   }
   else if ( len > size )
      return -1;
   else
      msgbuf = (char *)buf;
      
   size = len;
   
/* Read the message buffer contents */

   remainder = len % MAX_XMIT_SIZE;
   packets = (len - remainder) / MAX_XMIT_SIZE + 1;
   
   for ( ;packets;packets-- )
   {
      if ( packets == 1 ) len = remainder;
      else len = MAX_XMIT_SIZE;

      for( rcvd=0; rcvd<len; rcvd+=cc )
      {
         errno = 0;

         cc = read( fd, (char*)msgbuf + rcvd, len - rcvd );

         if ( cc == 0 ) return -1; /* EOF (i.e. socket closed) condition */
      
         if ( cc < 0 )
         {
            if ( errno == EINTR || errno == EAGAIN ) cc = 0;
            else return -1;
         }
      }
      msgbuf = (char *)msgbuf + MAX_XMIT_SIZE;
   }
   
   return size;
}

