#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

/*
This function opens a connection to a remote tcp service.

Returns: open file descriptor or
         -1 on error and error number stored in errno
         -2 on error and error number stored in h_errno
         -3 on other error
*/

int open_tcp_service( const char *ServerNode, const char *ServiceName )
{
int s, port;                    /* connected socket descriptor */
struct hostent *hp;	        /* pointer to host info for remote host */
struct servent *sp;             /* pointer to service information */
struct sockaddr_in myaddr_in;   /* for local socket address */
struct sockaddr_in peeraddr_in; /* for peer socket address */

/* Make TCPIP connection to EMU relay service */

   memset ( &myaddr_in, 0, sizeof(struct sockaddr_in) );
   memset ( &peeraddr_in, 0, sizeof(struct sockaddr_in) );
	
   hp = (struct hostent *)gethostbyname( ServerNode );
   if ( hp == NULL ) return -2;

   sp = (struct servent *)getservbyname( ServiceName, "tcp" );
   if ( sp == NULL )
   {
      port = atoi( ServiceName );
      if ( !port ) return -3;
      peeraddr_in.sin_port = htons(port);
   }
   else
      peeraddr_in.sin_port = sp->s_port;

   peeraddr_in.sin_family = hp->h_addrtype;
   memcpy( &peeraddr_in.sin_addr, hp->h_addr, sizeof(u_long) );

/* Create socket. Return on failure. */

   s = socket( hp->h_addrtype, SOCK_STREAM, 0 );             
   if ( s == -1 ) return -1;
   
/* Establish connection. Close socket and return on failure. */

   if ( connect(s, (struct sockaddr *)&peeraddr_in, sizeof(peeraddr_in)) == -1)
   {
      close(s);
      return -1;
   }

   return s;
}
